Manager = class(Turbine.UI.Window);

function Manager:Constructor()
    Turbine.UI.Window.Constructor(self);

    self.transfers = {
        [Turbine.ChatType.UserChat1] = {};
        [Turbine.ChatType.UserChat2] = {};
        [Turbine.ChatType.UserChat3] = {};
        [Turbine.ChatType.UserChat4] = {};
        [Turbine.ChatType.UserChat5] = {};
        [Turbine.ChatType.UserChat6] = {};
        [Turbine.ChatType.UserChat7] = {};
        [Turbine.ChatType.UserChat8] = {};
    };
    self.commands = {
        [Turbine.ChatType.UserChat1] = "/1";
        [Turbine.ChatType.UserChat2] = "/2";
        [Turbine.ChatType.UserChat3] = "/3";
        [Turbine.ChatType.UserChat4] = "/4";
        [Turbine.ChatType.UserChat4] = "/4";
        [Turbine.ChatType.UserChat5] = "/5";
        [Turbine.ChatType.UserChat6] = "/6";
        [Turbine.ChatType.UserChat7] = "/7";
        [Turbine.ChatType.UserChat8] = "/8";
    };
    self.canvases = {};
    self.nextCanvasID = 1;
    self.playerName = Turbine.Gameplay.LocalPlayer:GetInstance():GetName();
    self.pluginVersion = plugin:GetVersion();
    self.stripper = Turbine.UI.Label();
    AddCallback(Turbine.Chat, "Received", function(_, args)
        if (self.transfers[args.ChatType]) then
            self:Receive(args.ChatType, args.Message);
        end
    end);
    
    self.icon = Thurallor.UI.Icon("Thurallor/Blackboard/Images/icon.tga", self.canvases, { position = { 0.5, 0.5 } });
    self.icon:SetSize(48, 48);
    Thurallor.UI.Tooltip("Blackboard"):Attach(self.icon);
--    AddCallback(self.icon, "SettingsChanged", function()
--        self:SaveSettings();
--    end);
    
end

function Manager:SetChannelEnabled(channel, enable)
    if (enable) then
        self.transfers[channel] = self.transfers[channel] or {};
    else
        self.transfers[channel] = nil;
    end
end
    
function Manager:Receive(channel, message)
    -- Strip color codes, links, etc.
    self.stripper:SetText(message);
    message = self.stripper:GetText();

    -- Make sure it's addressed to us.
    local from, packetsLeft, data = message:match("^%[[^%]]+%] (.*): '<u>B</u>([0-9][0-9])(.*)'\n");
--Puts(Serialize({from, packetsLeft, data}));    
    if (not data) then
        return;
    end

    -- Start a new data transfer
    local transfer = self.transfers[channel][from];
    if (not transfer) then
--Puts("Receiving " .. (packetsLeft + 1) .. " packets of data from " .. from .. "...");
        transfer = { packetsLeft = tonumber(packetsLeft); packets = { data } };
        self.transfers[channel][from] = transfer;
    else
        -- Continuing a data transfer
        if ((transfer.packetsLeft - 1) ~= tonumber(packetsLeft)) then
Puts("Error: Expected packetsLeft = " .. (transfer.packetsLeft - 1) .. ", got " .. packetsLeft);
            return;
        end
        transfer.packetsLeft = tonumber(packetsLeft);
        table.insert(transfer.packets, data);
    end

    if (transfer.packetsLeft == 0) then
        data = table.concat(transfer.packets, "");
--Puts("Received a total of " .. data:len() .. " bytes from " .. from .. ".");
        self.transfers[channel][from] = nil;
        self:Process(self.commands[channel], from, data);
    end
end

function Manager:Process(command, from, data)
    -- Figure out which Canvas these changes are addressed to
    local properties = Shape.encoder:Decode(data);
    properties.Owner = properties.Owner or from;
    local id = properties.Owner .. properties.ID;

    local canvas = self.canvases[id];
    local newCanvas = not canvas;
    if (newCanvas) then
Puts(from .. " created a new blackboard: " .. properties.Text);
        canvas = Canvas();
        canvas:SetID(properties.ID);
        canvas:SetOwner(properties.Owner);
        local editor = canvas:SetEditorEnabled(true);
        editor:SetCommand(command);
        canvas:ResetChanges();
        self.canvases[id] = canvas;
        self.icon:SetWindows(self.canvases);
    end
    properties.Class = nil;
    canvas:ApplyChanges(properties, from);
    if (newCanvas) then
        CenterWindow(canvas);
    end
    DoCallbacks(canvas, "Changed"); -- tell editor to update sendButton
end

function Manager:CreateLocal(channel, title)
    self:SetChannelEnabled(channel, true);
    local canvas = Canvas();
    local id = self.playerName .. self.nextCanvasID;
    self.canvases[id] = canvas;
    self.icon:SetWindows(self.canvases);
    canvas:SetID(self.nextCanvasID);
    self.nextCanvasID = self.nextCanvasID + 1;
    canvas:SetOwner(self.playerName);
    canvas:SetVersion(self.pluginVersion);
    canvas:SetPosition(50, 50);
    canvas:SetText(title);
    local editor = canvas:SetEditorEnabled(true);
    editor:SetCommand(self.commands[Turbine.ChatType["UserChat" .. channel]]);

    -- Add title shape to canvas
    local textShape = editor:CreateTextShape(title);
    textShape.baseClass.SetTextAlignment(textShape, Turbine.UI.ContentAlignment.MiddleCenter);
    textShape:SetForeColor(Turbine.UI.Color.PaleGoldenrod);
    textShape:SetFont(Turbine.UI.Lotro.Font.TrajanPro20);
    textShape:SetMultiline(false);
    textShape:AutoSize();
    textShape:SetPosition(math.floor(0.5 + (canvas:GetWidth() - textShape:GetWidth()) / 2), 0);
    
    DoCallbacks(canvas, "Changed");
end