Icon = class(Turbine.UI.Window)

function Icon:Constructor(background, windows, settings)
    Turbine.UI.Window.Constructor(self);
    self.zoomers = {};
    self.hudVisible = true;
    self:SetSize(32, 32);
    self:SetBackground(background);
--    self:SetBlendMode(Turbine.UI.BlendMode.Overlay);
    self:SetVisible(true);
    self:SetOpacity(0.75);
    self:SetWantsKeyEvents(true); -- to respond to F12
    self:ApplySettings(settings);
    AddCallback(Turbine.UI.Display, "SizeChanged", function()
        self:SetFractionalPosition(self.x, self.y);
    end);
    self:SetWindows(windows);
end

function Icon:SetWindows(windows)
    if (windows.SetVisible) then
        -- A single window.  Put it in a table.
        windows = { windows };
    end
    for _, window in pairs(windows) do
        window.CloseButtonClicked = function(w)
            self.zoomers[w] = Thurallor.UI.Zoomer(w, self);
        end
    end
    self.windows = windows;
end

function Icon:KeyDown(args)
    if (args.Action == Turbine.UI.Lotro.Action.ToggleHUD) then
        self.hudVisible = not self.hudVisible;
        DoCallbacks(self, "HUDVisibleChanged", self.hudVisible);
        self:SetVisible(self.hudVisible);
    end
end

function Icon:SetPosition(left, top)
    self:SetPositionOnScreen(left, top);
    -- Update fractional screen coordinates
    local width, height = self:GetSize();
    local center = left + width / 2;
    local middle = top + height / 2;
    local screenWidth, screenHeight = Turbine.UI.Display:GetSize();
    self.x = center / screenWidth;
    self.y = middle / screenHeight;
end

-- like SetPosition(), but won't allow you to move the icon off-screen.
function Icon:SetPositionOnScreen(left, top)
    local screenWidth, screenHeight = Turbine.UI.Display:GetSize();
    local width, height = self:GetSize();
    margin = 0.1;
    left = math.max(0 - margin * width, left);
    top = math.max(0 - margin * height, top);
    left = math.min(screenWidth - width * (1 - margin), left);
    top = math.min(screenHeight - height * (1 - margin),  top);
    Turbine.UI.Window.SetPosition(self, left, top);
end    

function Icon:SetFractionalPosition(x, y)
    self.x, self.y = x, y;
    local width, height = self:GetSize();
    local screenWidth, screenHeight = Turbine.UI.Display:GetSize();
    local center = screenWidth * x;
    local middle = screenHeight * y;
    local left = math.floor(0.5 + center - width / 2);
    local top = math.floor(0.5 + middle - height / 2);
    self:SetPositionOnScreen(left, top);
end

function Icon:GetFractionalPosition()
    return self.x, self.y;
end

function Icon:DisplaySizeChanged()
    self:SetFractionalPosition(self.x, self.y);
    self:SetSize(self:GetSize()); -- workaround for bug that resets stretching when screen is resized
end

function Icon:Iconify(instantly)
    for _, window in pairs(self.windows) do
        if ((not instantly) and window:IsVisible()) then
            self.zoomers[window] = Thurallor.UI.Zoomer(window, self);
        end
        window:SetVisible(false);
    end
    self:Activate();
end


function Icon:Restore(instantly)
    for _, window in pairs(self.windows) do
        if (instantly) then
            window:SetVisible(true);
            window:Activate();
        else
            self.zoomers[window] = Thurallor.UI.Zoomer(self, window);
            self.zoomers[window].ZoomComplete = function()
                window:SetVisible(true);
                window:Activate();
            end
        end
    end
end

function Icon:MouseEnter()
    self.mouseInside = true;
    self:SetOpacity(1);
end

function Icon:MouseLeave()
    self.mouseInside = false;
    self:SetOpacity(0.75);
end

function Icon:MouseClick(args)
    if ((args.Button == Turbine.UI.MouseButton.Left) and not self.moved) then
    
        -- Find if any windows are visible.
        local visible = false;
        for _, window in pairs(self.windows) do
            if (window:IsVisible()) then
                visible = true;
                break;
            end
        end

        if (visible) then
            self:Iconify();
        else
            self:Restore();
        end
    end
end

function Icon:MouseDown(args)
    self:Activate(); -- bring to front
    if (args.Button == Turbine.UI.MouseButton.Left) then
        self.mouseDown = { Turbine.UI.Display:GetMousePosition() };
        self.moved = nil;
    end
end

function Icon:MouseMove()
    if (self.mouseDown) then
        self.moved = true;
        local oldX, oldY = unpack(self.mouseDown);
        local newX, newY = Turbine.UI.Display:GetMousePosition();
        local left, top = self:GetPosition();
        self:SetPosition(left + newX - oldX, top + newY - oldY);
        self.settings.position = { self:GetFractionalPosition() };
        self:SaveSettings();
        self.mouseDown = { newX, newY };
    end
end

function Icon:MouseUp(args)
    if (args.Button == Turbine.UI.MouseButton.Left) then
        if (self.moved) then
            DoCallbacks(self, "IconMoved");
        end
        self.mouseDown = nil;
    end
end

function Icon:SaveSettings()
    DoCallbacks(self, "SettingsChanged");
end

function Icon:ApplySettings(settings)
    self.settings = settings;
    self:SetFractionalPosition(unpack(settings.position));
end

Thurallor = Thurallor or {};
Thurallor.UI = Thurallor.UI or {};
Thurallor.UI.Icon = Icon;
